
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27, 16, 2); // I2C address 0x27, 16 column and 2 rows

#define UNKNOWN 0
#define BLUE 1
#define GREEN 2
#define RED 3
#define YELLOW 4
#define PINK 5
#define PURPLE 6


// Define color sensor pins
#define S0 4
#define S1 2
#define S2 13
#define S3 7
#define SENSOR_OUT 8

// Define RGB LED pins
#define RED_PIN PD3
#define GREEN_PIN PD5
#define BLUE_PIN PD6

#define BUTTON PB3

#define DEBOUNCE_TIME 50
long int last_press = 0;


// Variables for color pulse width
int redPW = 0;
int greenPW = 0;
int bluePW = 0;

void USART0_init()
{
  /* seteaza baud rate la 9600 */
  UBRR0 = 103;
 
  /* porneste transmitatorul */
  UCSR0B = (1 << TXEN0) | (1 << RXEN0);
 
  /* seteaza formatul frame-ului: 8 biti de date, 1 bit de stop, fara paritate */
  UCSR0C &= ~(1 << USBS0);
  UCSR0C |= (3 << UCSZ00);
}

void USART0_transmit(unsigned char* data) {
  for (size_t i = 0; data[i] != 0; i++) {
    while(!(UCSR0A & (1<<UDRE0)));
    UDR0 = data[i];
  }
}


int checkColor(int redPW, int greenPW, int bluePW) {

  // for blue:
  if (redPW <= 87 && redPW >= 66 && greenPW <= 50 && greenPW >= 32 && bluePW <= 34 && bluePW >= 16) {
    return BLUE;
  }

  // for green:
  if (redPW <= 89 && redPW >= 58 && greenPW <= 65 && greenPW >= 39 && bluePW <= 70 && bluePW >= 44) {
    return GREEN;
  }

  // for red
  if (redPW <= 30 && redPW >= 12 && greenPW <= 85 && greenPW >= 67 && bluePW <= 66 && bluePW >= 48) {
    return RED;
  }

  // for yellow
  if (redPW <= 23 && redPW >= 5 && greenPW <= 34 && greenPW >= 16 && bluePW <= 49 && bluePW >= 31) {
    return YELLOW;
  }

  // for pink
  if (redPW <= 30 && redPW >= 12 && greenPW <= 84 && greenPW >= 64 && bluePW <= 46 && bluePW >= 28) {
    return PINK;
  }

  // for purple
  if (redPW <= 100 && redPW >= 65 && greenPW <= 108 && greenPW >= 74 && bluePW <= 75 && bluePW >= 45) {
    return PURPLE;
  }

  return UNKNOWN;
}

ISR(PCINT0_vect) {

  if (PINB & (1 << BUTTON))
    return;

  if (millis() - last_press < DEBOUNCE_TIME)
    return;

  last_press = millis();

  // Read Red value
  readRedPW();

  // Read Green value
  readGreenPW();

  // Read Blue value
  readBluePW();

  
}

void setup() {
  // put your setup code here, to run once:
  USART0_init();

  pinMode(S0, OUTPUT);
  pinMode(S1, OUTPUT);
  pinMode(S2, OUTPUT);
  pinMode(S3, OUTPUT);

  DDRB &= ~(1 << BUTTON);
  PORTB |= (1 << BUTTON);

  PCICR |= (1 << PCIE0);
  PCMSK0 |= (1 << PCINT3);

  pinMode(SENSOR_OUT, INPUT);

  // Set Frequency scaling to 20%
  digitalWrite(S0, HIGH);
  digitalWrite(S1, LOW);

  // Set RGB LED pins as outputs
  DDRD |= (1 << RED_PIN);
  DDRD |= (1 << GREEN_PIN);
  DDRD |= (1 << BLUE_PIN);

  lcd.init(); // initialize the lcd
  lcd.backlight();
  lcd.clear();


}

void loop() {

  delay(2000);
  switch (checkColor(redPW, greenPW, bluePW)) {
    case BLUE:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Color: BLUE");
      analogWrite(RED_PIN, 0);
      analogWrite(GREEN_PIN, 0);
      analogWrite(BLUE_PIN, 255);
      USART0_transmit("BLUE\n");
      break;

    case GREEN:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Color: GREEN");
      analogWrite(RED_PIN, 0);
      analogWrite(GREEN_PIN, 255);
      analogWrite(BLUE_PIN, 0);
      USART0_transmit("GREEN\n");
      break;
    
    case RED:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Color: RED");
      analogWrite(RED_PIN, 255);
      analogWrite(GREEN_PIN, 0);
      analogWrite(BLUE_PIN, 0);
      USART0_transmit("RED\n");
      break;

    case YELLOW:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Color: YELLOW");
      analogWrite(RED_PIN, 255);
      analogWrite(GREEN_PIN, 255);
      analogWrite(BLUE_PIN, 0);
      USART0_transmit("YELLOW\n");
      break;
    
    case PINK:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Color: PINK");
      analogWrite(RED_PIN, 255);
      analogWrite(GREEN_PIN, 16);
      analogWrite(BLUE_PIN, 240);
      USART0_transmit("PINK\n");
      break;

    case PURPLE:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Color: PURPLE");
      analogWrite(RED_PIN, 160);
      analogWrite(GREEN_PIN, 32);
      analogWrite(BLUE_PIN, 240);
      USART0_transmit("PURPLE\n");
      break;
    
    default:
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Color: UNKNOWN");
      analogWrite(RED_PIN, 255);
      analogWrite(GREEN_PIN, 255);
      analogWrite(BLUE_PIN, 255);
      USART0_transmit("UNKNOWN\n");
  }

}

int readRedPW()
{
  // Set sensor to read Red only (as mentioned in datasheet)
  digitalWrite(S2, LOW);
  digitalWrite(S3, LOW);

  // Read the output Pulse Width
  redPW = pulseIn(SENSOR_OUT, LOW);
}

int readGreenPW()
{
  // Set sensor to read green only (as mentioned in datasheet)
  digitalWrite(S2, HIGH);
  digitalWrite(S3, HIGH);

  // Read the output pulse width
  greenPW = pulseIn(SENSOR_OUT, LOW);
}

int readBluePW() {

  // Set sensor to read blue only (as mentioned in datasheet)
  digitalWrite(S2, LOW);
  digitalWrite(S3, HIGH);

  // Read the output pulse width
  bluePW = pulseIn(SENSOR_OUT, LOW);
}
